@@ -555,6 +555,8 @@ cdb_sql_exec (sqlite3 *db,
 	gchar *errmsg = NULL;
 	gint   ret = -1, retries = 0;
 
+	g_return_val_if_fail (stmt != NULL, -1);
+
 	d (g_print ("Camel SQL Exec:\n%s\n", stmt));
 
 	ret = sqlite3_exec (db, stmt, callback, data, &errmsg);
@@ -564,10 +566,7 @@ cdb_sql_exec (sqlite3 *db,
 			break;
 		retries++;
 
-		if (errmsg) {
-			sqlite3_free (errmsg);
-			errmsg = NULL;
-		}
+		g_clear_pointer (&errmsg, sqlite3_free);
 		g_thread_yield ();
 		g_usleep (100 * 1000); /* Sleep for 100 ms */
 
@@ -587,10 +586,7 @@ cdb_sql_exec (sqlite3 *db,
 		return -1;
 	}
 
-	if (errmsg) {
-		sqlite3_free (errmsg);
-		errmsg = NULL;
-	}
+	g_clear_pointer (&errmsg, sqlite3_free);
 
 	return 0;
 }
@@ -1086,6 +1082,7 @@ camel_db_add_to_transaction (CamelDB *cdb,
 		return -1;
 
 	g_return_val_if_fail (cdb_is_in_transaction (cdb), -1);
+	g_return_val_if_fail (query != NULL, -1);
 
 	return (cdb_sql_exec (cdb->priv->db, query, NULL, NULL, NULL, error));
 }
@@ -1176,6 +1173,8 @@ camel_db_count_message_info (CamelDB *cdb,
 {
 	gint ret = -1;
 
+	g_return_val_if_fail (query != NULL, -1);
+
 	cdb_reader_lock (cdb);
 
 	START (query);
@@ -1447,6 +1446,8 @@ camel_db_select (CamelDB *cdb,
 	if (!cdb)
 		return ret;
 
+	g_return_val_if_fail (stmt != NULL, ret);
+
 	d (g_print ("\n%s:\n%s \n", G_STRFUNC, stmt));
 	cdb_reader_lock (cdb);
 
@@ -2884,7 +2885,7 @@ get_number_cb (gpointer data,
 /**
  * camel_db_maybe_run_maintenance:
  * @cdb: a #CamelDB
- * @error: (allow-none): a #GError or %NULL
+ * @error: a #GError or %NULL
  *
  * Runs a @cdb maintenance, which includes vacuum, if necessary.
  *
