@@ -62,6 +62,7 @@ enum {
 	COL_SUPPORTS_STRING,
 	COL_COLOR_GDKRGBA,
 	COL_SHOW_COLOR_BOOLEAN,
+	COL_ORDER_UINT,
 	N_COLUMNS
 };
 
@@ -103,8 +104,8 @@ e_webdav_discover_content_init (EWebDAVDiscoverContent *self)
 /**
  * e_webdav_discover_content_new:
  * @credentials_prompter: an #ECredentialsPrompter to use to ask for credentials
- * @source: (allow-none): optional #ESource to use for authentication, or %NULL
- * @base_url: (allow-none): optional base URL to use for discovery, or %NULL
+ * @source: (nullable): optional #ESource to use for authentication, or %NULL
+ * @base_url: (nullable): optional base URL to use for discovery, or %NULL
  * @supports_filter: a bit-or of #EWebDAVDiscoverSupports, a filter to limit what source
  *    types will be shown in the dialog content; use %E_WEBDAV_DISCOVER_SUPPORTS_NONE
  *    to show all
@@ -151,7 +152,8 @@ e_webdav_discover_content_new (ECredentialsPrompter *credentials_prompter,
 					 G_TYPE_STRING, /* COL_DESCRIPTION_STRING */
 					 G_TYPE_STRING, /* COL_SUPPORTS_STRING */
 					 GDK_TYPE_RGBA, /* COL_COLOR_GDKRGBA */
-					 G_TYPE_BOOLEAN); /* COL_SHOW_COLOR_BOOLEAN */
+					 G_TYPE_BOOLEAN,/* COL_SHOW_COLOR_BOOLEAN */
+					 G_TYPE_UINT);  /* COL_ORDER_UINT */
 
 	tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
 	g_object_unref (list_store);
@@ -359,6 +361,7 @@ e_webdav_discover_content_get_base_url (GtkWidget *content)
  * @out_display_name: (out): an output location of the sources display name
  * @out_color: (out): an output location of the string representation of the color
  *    for the source, as set on the server
+ * @out_order: (out): an output location of the preferred sorting order
  *
  * Returns information about selected source at index @index. The function can be called
  * multiple times, with the index starting at zero and as long as it doesn't return %FALSE.
@@ -377,7 +380,8 @@ e_webdav_discover_content_get_selected (GtkWidget *content,
 					gchar **out_href,
 					guint *out_supports,
 					gchar **out_display_name,
-					gchar **out_color)
+					gchar **out_color,
+					guint *out_order)
 {
 	EWebDAVDiscoverContent *self;
 	GtkTreeSelection *selection;
@@ -413,6 +417,7 @@ e_webdav_discover_content_get_selected (GtkWidget *content,
 					COL_SUPPORTS_UINT, out_supports,
 					COL_DISPLAY_NAME_STRING, out_display_name,
 					COL_COLOR_STRING, out_color,
+					COL_ORDER_UINT, out_order,
 					-1);
 			}
 		}
@@ -547,6 +552,7 @@ e_webdav_discover_content_fill_discovered_sources (GtkTreeView *tree_view,
 			COL_SUPPORTS_UINT, source->supports,
 			COL_DISPLAY_NAME_STRING, source->display_name,
 			COL_COLOR_STRING, colorstr,
+			COL_ORDER_UINT, source->order,
 			COL_DESCRIPTION_STRING, description_markup,
 			COL_SUPPORTS_STRING, supports->str,
 			COL_COLOR_GDKRGBA, show_color ? &rgba : NULL,
@@ -802,11 +808,10 @@ e_webdav_discover_info_bar_response_cb (GtkInfoBar *info_bar,
 /**
  * e_webdav_discover_content_refresh:
  * @content: (type EWebDAVDiscoverContent): an #EWebDAVDiscoverContent
- * @display_name: (allow-none): optional display name to use for scratch sources
- * @cancellable: (allow-none): optional #GCancellable object, or %NULL
- * @callback: (scope async): a #GAsyncReadyCallback to call when the request
- *            is satisfied
- * @user_data: (closure): data to pass to the callback function
+ * @display_name: (nullable): optional display name to use for scratch sources
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when the request is satisfied
+ * @user_data: data to pass to the callback function
  *
  * Asynchronously starts refresh of the @content. This means to access the server
  * and search it for available sources. The @content shows a feedback and a Cancel
@@ -923,7 +928,7 @@ e_webdav_discover_content_refresh (GtkWidget *content,
  * e_webdav_discover_content_refresh_finish:
  * @content: (type EWebDAVDiscoverContent): an #EWebDAVDiscoverContent
  * @result: a #GAsyncResult
- * @error: (allow-none): return location for a #GError, or %NULL
+ * @error: return location for a #GError, or %NULL
  *
  * Finishes the operation started with e_webdav_discover_content_refresh(). If an
  * error occurred, the function will set @error and return %FALSE. There is
@@ -965,7 +970,7 @@ e_webdav_discover_info_bar_error_response_cb (GtkInfoBar *info_bar,
 /**
  * e_webdav_discover_content_show_error:
  * @content: (type EWebDAVDiscoverContent): an #EWebDAVDiscoverContent
- * @error: (allow-none): a #GError to show in the UI, or %NULL
+ * @error: a #GError to show in the UI, or %NULL
  *
  * Shows the @error within @content, unless it's a #G_IO_ERROR_CANCELLED, or %NULL,
  * which are safely ignored. The advantage of this function is that the error
@@ -1050,7 +1055,7 @@ e_webdav_discover_dialog_init (EWebDAVDiscoverDialog *self)
  * @title: title of the window
  * @credentials_prompter: an #ECredentialsPrompter to use to ask for credentials
  * @source: an #ESource to use for authentication
- * @base_url: (allow-none): optional base URL to use for discovery, or %NULL
+ * @base_url: (nullable): optional base URL to use for discovery, or %NULL
  * @supports_filter: a bit-or of #EWebDAVDiscoverSupports, a filter to limit what source
  *    types will be shown in the dialog content; use %E_WEBDAV_DISCOVER_SUPPORTS_NONE
  *    to show all
