@@ -117,5 +117,5 @@ main (gint argc,
 		test_get_revision,
 		e_test_server_utils_teardown);
 
-	return e_test_server_utils_run ();
+	return e_test_server_utils_run (argc, argv);
 }
